import 'dart:async';

import 'package:ai_app/globals/global_functions.dart';
import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:ai_app/globals/global_colors.dart';
import 'package:ai_app/globals/global_variables.dart' as global_variables;
import 'package:ai_app/screens/interview_screen.dart';
import 'package:ai_app/widgets/ai_outlined_button.dart';

class IntroductionScreen extends StatefulWidget {
  @override
  _IntroductionScreenState createState() => _IntroductionScreenState();

  static const timeout = Duration(seconds: 10);
  static const ms = Duration(milliseconds: 1);
}

class _IntroductionScreenState extends State<IntroductionScreen> {
  var firstName;
  var lastName;

  int screenNum = 1;

  late String message1;

  late String message2;

  late String message3;

  late String message4;

  late String message5;

  late String message6;

  void initState() {
    message1 =
        "Welcome to the Phone.do Interview. \nMy name is Jenny and I will ask you \nquestions during all interview. \n ";
    message2 =
        "Fill free to asked me anything in case \nyou did not understand me or if you \nhave any question during the \ninterview";
    message3 =
        "during the interview, a chat room will \nbe opened, so you can also send me \ntext messages and I will answer you \nback. ";
    message4 = "Ok, " +
        global_variables.currentUser.firstName.toString() +
        ", I’m done introducing myself. In about 3 seconds a “Record” button will appear, So feel free to click on it when you ready to start.";
    message5 =
        "Unfortunately, you stopped the \ninterview in the middle so I can not \nallow you to continue using our \nservice, thanks, and good luck!";
    message6 = "Thank you " +
        global_variables.currentUser.firstName.toString() +
        ", for participating in \nour interview. We will go over the data \nand get back to you soon.";
    startTimeout(10000);

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(
        global_variables.screenWidth,
        global_variables.screenHeight,
      ),
      builder: () => Scaffold(
        body: _buildContent(),
      ),
    );
  }

  _buildContent() {
    return Column(
      children: [
        Expanded(
          child: Column(
            children: [
              screenNum == 5
                  ? Text("")
                  : _buildTitle(
                      text: screenNum == 8
                          ? "Thank You!"
                          : "Hello, " +
                              global_variables.currentUser.firstName
                                  .toString() +
                              " " +
                              global_variables.currentUser.lastName.toString(),
                    ),
              _buildMain(),
            ],
          ),
        ),
        _buildDoll(),
      ],
    );
  }

  _buildTitle({required String text}) {
    return Padding(
      padding: EdgeInsets.only(
        top: GlobalFunctions().setHeight(context, 105),
      ),
      child: Center(
        child: Text(
          text,
          style: TextStyle(
            fontSize: ScreenUtil().setSp(24),
            color: GlobalColors.aiBlue,
          ),
        ),
      ),
    );
  }

  _buildMain() {
    return Padding(
      padding: EdgeInsets.only(
        top: GlobalFunctions().setHeight(context, 39),
      ),
      child: Center(
        child: screenNum == 5
            ? _buildRecordButton()
            : screenNum == 7
                ? _buildDone()
                : screenNum == 8
                    ? _buildStop()
                    : _buildMessageItem(),
      ),
    );
  }

  _buildMessageItem() {
    return Column(
      children: [
        Container(
          width: GlobalFunctions().setWidth(context, 264),
          decoration: BoxDecoration(
            color: GlobalColors.aiLightGrey,
            borderRadius: BorderRadius.circular(10),
          ),
          child: Padding(
            padding: EdgeInsets.only(
              top: GlobalFunctions().setHeight(context, 14),
              left: GlobalFunctions().setWidth(context, 13),
              bottom: GlobalFunctions().setHeight(context, 18),
            ),
            child: Text(
              screenNum == 1
                  ? message1
                  : screenNum == 2
                      ? message2
                      : screenNum == 3
                          ? message3
                          : screenNum == 4
                              ? message4
                              : screenNum == 6
                                  ? message5
                                  : screenNum == 8
                                      ? message6
                                      : "",
              style: TextStyle(
                fontSize: ScreenUtil().setSp(14),
                height: screenNum == 8
                    ? ScreenUtil().setSp(1.4)
                    : ScreenUtil().setSp(1.7),
                color: GlobalColors.aiBlue,
              ),
            ),
          ),
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 5),
            left: GlobalFunctions().setWidth(context, 258.76),
          ),
          child: Container(
            decoration: new BoxDecoration(
              color: GlobalColors.aiLightGrey,
              shape: BoxShape.circle,
            ),
            height: GlobalFunctions().setHeight(context, 11),
            width: GlobalFunctions().setWidth(context, 11),
          ),
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 5),
            left: GlobalFunctions().setWidth(context, 247),
          ),
          child: Container(
            decoration: new BoxDecoration(
              color: GlobalColors.aiLightGrey,
              shape: BoxShape.circle,
            ),
            height: GlobalFunctions().setHeight(context, 8.25),
            width: GlobalFunctions().setWidth(context, 8.25),
          ),
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 5),
            left: GlobalFunctions().setWidth(context, 236),
          ),
          child: Container(
            decoration: new BoxDecoration(
              color: GlobalColors.aiLightGrey,
              shape: BoxShape.circle,
            ),
            height: GlobalFunctions().setHeight(context, 5.5),
            width: GlobalFunctions().setWidth(context, 5.5),
          ),
        ),
      ],
    );
  }

  _buildRecordButton() {
    return Padding(
      padding: EdgeInsets.only(
        top: GlobalFunctions().setHeight(context, 109),
      ),
      child: Column(
        children: [
          Container(
            width: GlobalFunctions().setWidth(context, 80),
            height: GlobalFunctions().setHeight(context, 80),
            decoration: BoxDecoration(
              shape: BoxShape.circle,
              boxShadow: [
                BoxShadow(
                  color: Colors.grey.withOpacity(0.5),
                  spreadRadius: 4,
                  blurRadius: 10,
                  offset: Offset(0, 0),
                )
              ],
            ),
            child: RawMaterialButton(
              onPressed: () {
                _onRecordButtonPressed();
              },
              fillColor: Colors.white,
              child: Padding(
                padding: EdgeInsets.all(
                  GlobalFunctions().setHeight(context, 26),
                ),
                child: Container(
                  decoration: BoxDecoration(
                    color: Colors.red,
                    shape: BoxShape.circle,
                  ),
                ),
              ),
              shape: CircleBorder(),
            ),
          ),
          Padding(
            padding: EdgeInsets.only(
              top: GlobalFunctions().setHeight(context, 26),
            ),
            child: Text(
              "Start recording...",
              style: TextStyle(
                fontSize: ScreenUtil().setSp(20),
                color: GlobalColors.aiBlue,
              ),
            ),
          ),
        ],
      ),
    );
  }

  _buildDoll() {
    return Expanded(
      child: Container(
        child: Image.network(
          'https://res.cloudinary.com/feed-phonedo/image/upload/v1623825359/doll_b3aubv.png',
        ),
      ),
    );
  }

  _buildStop() {
    return Column(
      children: [
        AiOutlinedButton(
          onPressed: _onRecordButtonPressed,
          text: "Start a new interview",
          isGreenButton: true,
          isBlueFont: false,
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 24),
          ),
          child: AiOutlinedButton(
            onPressed: () {
              setState(() {
                screenNum = 0;
              });
              handleTimeout();
            },
            text: "Logout",
            isGreenButton: false,
            isBlueFont: true,
          ),
        ),
      ],
    );
  }

  _buildDone() {
    return Column(
      children: [
        AiOutlinedButton(
          onPressed: () {
            setState(() {
              screenNum = 0;
            });
            handleTimeout();
          },
          text: "Done",
          isGreenButton: true,
          isBlueFont: false,
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 41),
          ),
          child: _buildMessageItem(),
        ),
      ],
    );
  }

  Timer startTimeout([int? milliseconds]) {
    var duration = milliseconds == null
        ? IntroductionScreen.timeout
        : IntroductionScreen.ms * milliseconds;
    return Timer(duration, handleTimeout);
  }

  void handleTimeout() {
    if (screenNum < 5) {
      setState(() {
        screenNum++;
      });
      startTimeout(10000);
    }
  }

  void _onRecordButtonPressed() async {
    try {
      WidgetsFlutterBinding.ensureInitialized();
      global_variables.cameras = await availableCameras();
    } on CameraException catch (e) {
      logError(
        e.code,
        e.description,
      );
    }

    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => InterviewScreen(),
      ),
    ).then(
      (value) => setState(
        () {
          screenNum = 8;
        },
      ),
    );
  }
}
